@@ -53,12 +53,14 @@ update_state (GtkWidget *window)
                 break;
         case 3:
                 gsd_osd_window_set_volume_muted (GSD_OSD_WINDOW (window), TRUE);
+                gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (window), 0);
                 gsd_osd_window_set_action (GSD_OSD_WINDOW (window),
                                            GSD_OSD_WINDOW_ACTION_VOLUME);
 
                 gtk_widget_show (window);
                 break;
         case 4:
+                gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (window), NULL);
                 gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (window),
                                                   "media-eject-symbolic",
                                                   FALSE);
@@ -84,17 +86,33 @@ update_state (GtkWidget *window)
         case 7:
                 gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (window),
                                                   "touchpad-disabled-symbolic",
-                                                  TRUE);
+                                                  FALSE);
 
                 gtk_widget_show (window);
                 break;
         case 8:
                 gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (window),
                                                   "input-touchpad-symbolic",
-                                                  TRUE);
+                                                  FALSE);
 
                 gtk_widget_show (window);
                 break;
+
+        case 9: {
+                GIcon *icon;
+
+                icon = g_icon_new_for_string (". GThemedIcon battery-full-charging-symbolic gpm-battery-100-charging battery-full-charging ",
+                                              NULL);
+
+                gsd_osd_window_set_action_custom_gicon (GSD_OSD_WINDOW (window),
+                                                        icon,
+                                                        FALSE);
+
+                g_object_unref (icon);
+
+                gtk_widget_show (window);
+                break;
+                }
         default:
                 gtk_main_quit ();
                 break;
@@ -110,6 +128,7 @@ test_window (void)
 
         window = gsd_osd_window_new ();
         gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER_ALWAYS);
+        gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (window), "Headphones");
 
         gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (window), 0);
         gsd_osd_window_set_action (GSD_OSD_WINDOW (window),
@@ -136,9 +155,6 @@ main (int    argc,
                 exit (1);
         }
 
-        gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
-                                           DATADIR G_DIR_SEPARATOR_S "gnome-power-manager" G_DIR_SEPARATOR_S "icons");
-
         test_window ();
 
         gtk_main ();
