@@ -42,6 +42,7 @@
 #define MAX_FAILED_GET_UPDATES              10 /* the maximum number of tries */
 #define GSD_UPDATES_ICON_NORMAL             "software-update-available-symbolic"
 #define GSD_UPDATES_ICON_URGENT             "software-update-urgent-symbolic"
+#define GSD_UPDATES_CHECK_OFFLINE_TIMEOUT   30 /* time in seconds */
 
 struct GsdUpdatesManagerPrivate
 {
@@ -88,13 +89,13 @@ clear_offline_updates_message (void)
         gchar *argv[3];
         GPid pid;
 
-        argv[0] = BINDIR "/pkexec";
+        argv[0] = "pkexec";
         argv[1] = LIBEXECDIR "/pk-clear-offline-update";
         argv[2] = NULL;
         ret = g_spawn_async (NULL,
                              argv,
                              NULL,
-                             G_SPAWN_DO_NOT_REAP_CHILD,
+                             G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
                              NULL,
                              NULL,
                              &pid,
@@ -1332,7 +1333,7 @@ gsd_updates_manager_start (GsdUpdatesManager *manager,
 
         /* check for offline update */
         manager->priv->offline_update_id =
-                g_timeout_add_seconds (5,
+                g_timeout_add_seconds (GSD_UPDATES_CHECK_OFFLINE_TIMEOUT,
                                        check_offline_update_cb,
                                        manager);
 
